@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2009-2013 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2009-2014 B.A.T.M.A.N. contributors:
  *
- * Marek Lindner <lindner_marek@yahoo.de>
+ * Marek Lindner <mareklindner@neomailbox.ch>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,14 +19,15 @@
  *
  */
 
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdint.h>
-#include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <unistd.h>
+#include <stddef.h>
+#include <netinet/ether.h>
 
-#include "main.h"
 #include "bisect_iv.h"
 #include "bat-hosts.h"
 #include "hash.h"
