@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2009-2013 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2009-2014 B.A.T.M.A.N. contributors:
  *
- * Marek Lindner <lindner_marek@yahoo.de>
+ * Marek Lindner <mareklindner@neomailbox.ch>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -23,15 +23,11 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-#include <errno.h>
-#include <sys/types.h>
-#include <dirent.h>
 
-#include "main.h"
 #include "debug.h"
 #include "debugfs.h"
 #include "functions.h"
+#include "sys.h"
 
 const struct debug_table_data batctl_debug_tables[BATCTL_TABLE_NUM] = {
 	{
@@ -84,7 +80,7 @@ const struct debug_table_data batctl_debug_tables[BATCTL_TABLE_NUM] = {
 	},
 };
 
-void debug_table_usage(int debug_table)
+static void debug_table_usage(int debug_table)
 {
 	fprintf(stderr, "Usage: batctl [options] %s|%s [parameters]\n",
 	       batctl_debug_tables[debug_table].opt_long, batctl_debug_tables[debug_table].opt_short);
@@ -94,8 +90,10 @@ void debug_table_usage(int debug_table)
 	fprintf(stderr, " \t -H don't show the header\n");
 	fprintf(stderr, " \t -w [interval] watch mode - refresh the table continuously\n");
 
-	if (debug_table == BATCTL_TABLE_ORIGINATORS)
+	if (debug_table == BATCTL_TABLE_ORIGINATORS) {
 		fprintf(stderr, " \t -t timeout interval - don't print originators not seen for x.y seconds \n");
+		fprintf(stderr, " \t -i [interface] - show multiif originator table for a specific interface\n");
+	}
 }
 
 int handle_debug_table(char *mesh_iface, int debug_table, int argc, char **argv)
@@ -103,11 +101,12 @@ int handle_debug_table(char *mesh_iface, int debug_table, int argc, char **argv)
 	int optchar, read_opt = USE_BAT_HOSTS;
 	char full_path[MAX_PATH+1];
 	char *debugfs_mnt;
+	char *orig_iface = NULL;
 	float orig_timeout;
 	float watch_interval = 1;
 	opterr = 0;
 
-	while ((optchar = getopt(argc, argv, "hnw:t:H")) != -1) {
+	while ((optchar = getopt(argc, argv, "hnw:t:Hi:")) != -1) {
 		switch (optchar) {
 		case 'h':
 			debug_table_usage(debug_table);
@@ -143,11 +142,24 @@ int handle_debug_table(char *mesh_iface, int debug_table, int argc, char **argv)
 		case 'H':
 			read_opt |= SKIP_HEADER;
 			break;
-		case '?':
-			if (optopt == 't')
-				fprintf(stderr, "Error - option '-t' needs a number as argument\n");
+		case 'i':
+			if (debug_table != BATCTL_TABLE_ORIGINATORS) {
+				fprintf(stderr, "Error - unrecognised option '-%c'\n", optchar);
+				debug_table_usage(debug_table);
+				return EXIT_FAILURE;
+			}
 
-			else if (optopt == 'w') {
+			if (check_mesh_iface_ownership(mesh_iface, optarg) != EXIT_SUCCESS)
+				return EXIT_FAILURE;
+
+			orig_iface = optarg;
+			break;
+		case '?':
+			if (optopt == 't') {
+				fprintf(stderr, "Error - option '-t' needs a number as argument\n");
+			} else if (optopt == 'i') {
+				fprintf(stderr, "Error - option '-i' needs an interface as argument\n");
+			} else if (optopt == 'w') {
 				read_opt |= CLR_CONT_READ;
 				break;
 			}
@@ -167,12 +179,41 @@ int handle_debug_table(char *mesh_iface, int debug_table, int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	debugfs_make_path(DEBUG_BATIF_PATH_FMT "/", mesh_iface, full_path, sizeof(full_path));
+	if (orig_iface)
+		debugfs_make_path(DEBUG_BATIF_PATH_FMT "/", orig_iface, full_path, sizeof(full_path));
+	else
+		debugfs_make_path(DEBUG_BATIF_PATH_FMT "/", mesh_iface, full_path, sizeof(full_path));
 	return read_file(full_path, (char *)batctl_debug_tables[debug_table].debugfs_name,
 			 read_opt, orig_timeout, watch_interval,
 			 batctl_debug_tables[debug_table].header_lines);
 }
 
+int print_vis_info(char *mesh_iface)
+{
+	char full_path[MAX_PATH+1];
+	char *debugfs_mnt;
+	FILE *fp;
+
+	debugfs_mnt = debugfs_mount(NULL);
+	if (!debugfs_mnt) {
+		fprintf(stderr, "Error - can't mount or find debugfs\n");
+		return -1;
+	}
+
+	debugfs_make_path(DEBUG_BATIF_PATH_FMT "/vis_data", mesh_iface, full_path, sizeof(full_path));
+	fp = fopen(full_path, "r");
+	if (fp) {
+		fclose(fp);
+		fprintf(stderr, "Error - batctl version is newer than kernel module - the kernel module still supports\n"
+				"vis, but later versions will not. The vis functionality has been moved to the userspace\n"
+				"daemon ''alfred''. Please either downgrade to an older (compatible) batctl version or use alfred.\n");
+	} else {
+		fprintf(stderr, "Error - The installed batctl version and kernel module don't have vis support. The vis functionality\n"
+				"has been moved to the userspace daemon ''alfred''.\n");
+	}
+	return 0;
+}
+
 static void log_usage(void)
 {
 	fprintf(stderr, "Usage: batctl [options] log [parameters]\n");
