@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007-2013 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2014 B.A.T.M.A.N. contributors:
  *
- * Andreas Langer <an.langer@gmx.de>, Marek Lindner <lindner_marek@yahoo.de>
+ * Andreas Langer <an.langer@gmx.de>, Marek Lindner <mareklindner@neomailbox.ch>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,9 +19,11 @@
  *
  */
 
+#ifndef _BATCTL_FUNCTIONS_H
+#define _BATCTL_FUNCTIONS_H
 
-
-#include <netinet/if_ether.h>
+#include <net/ethernet.h>
+#include <stddef.h>
 
 
 #define ETH_STR_LEN 17
@@ -34,11 +36,13 @@ char *ether_ntoa_long(const struct ether_addr *addr);
 char *get_name_by_macaddr(struct ether_addr *mac_addr, int read_opt);
 char *get_name_by_macstr(char *mac_str, int read_opt);
 int file_exists(const char *fpath);
-int read_file(char *dir, char *path, int read_opt,
+int read_file(const char *dir, const char *path, int read_opt,
 	      float orig_timeout, float watch_interval, size_t header_lines);
-int write_file(char *dir, char *fname, char *arg1, char *arg2);
+int write_file(const char *dir, const char *fname, const char *arg1,
+	       const char *arg2);
 struct ether_addr *translate_mac(char *mesh_iface, struct ether_addr *mac);
 struct ether_addr *resolve_mac(const char *asc);
+int vlan_get_link(const char *ifname, char **parent);
 
 extern char *line_ptr;
 
@@ -54,3 +58,5 @@ enum {
 	COMPAT_FILTER = 0x80,
 	SKIP_HEADER = 0x100,
 };
+
+#endif
