@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007-2013 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2014 B.A.T.M.A.N. contributors:
  *
- * Andreas Langer <an.langer@gmx.de>, Marek Lindner <lindner_marek@yahoo.de>
+ * Andreas Langer <an.langer@gmx.de>, Marek Lindner <mareklindner@neomailbox.ch>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,8 +19,11 @@
  *
  */
 
+#ifndef _BATCTL_MAIN_H
+#define _BATCTL_MAIN_H
+
 #ifndef SOURCE_VERSION
-#define SOURCE_VERSION "2013.4.0"
+#define SOURCE_VERSION "2014.1.0"
 #endif
 
 #define SOCKET_PATH_FMT "%s/batman_adv/%s/socket"
@@ -33,7 +36,17 @@
 #define DEBUG_TABLE_PATH_MAX_LEN 20
 #define SETTINGS_PATH_MAX_LEN 25
 
+#if BYTE_ORDER == BIG_ENDIAN
+#define __BIG_ENDIAN_BITFIELD
+#elif BYTE_ORDER == LITTLE_ENDIAN
+#define __LITTLE_ENDIAN_BITFIELD
+#else
+#error "unknown endianess"
+#endif
+
 #define __packed __attribute((packed))   /* linux kernel compat */
 #define BIT(nr)                 (1UL << (nr)) /* linux kernel compat */
 
 extern char module_ver_path[];
+
+#endif
