@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007-2013 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2014 B.A.T.M.A.N. contributors:
  *
- * Andreas Langer <an.langer@gmx.de>, Marek Lindner <lindner_marek@yahoo.de>
+ * Andreas Langer <an.langer@gmx.de>, Marek Lindner <mareklindner@neomailbox.ch>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -30,6 +30,11 @@
 #include <fcntl.h>
 #include <string.h>
 #include <math.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <sys/select.h>
+#include <sys/time.h>
+#include <linux/if_ether.h>
 
 #include "main.h"
 #include "ping.h"
@@ -42,7 +47,7 @@
 char is_aborted = 0;
 
 
-void ping_usage(void)
+static void ping_usage(void)
 {
 	fprintf(stderr, "Usage: batctl [options] ping [parameters] mac|bat-host|host_name|IPv4_address \n");
 	fprintf(stderr, "parameters:\n");
@@ -54,7 +59,7 @@ void ping_usage(void)
 	fprintf(stderr, " \t -T don't try to translate mac to originator address\n");
 }
 
-void sig_handler(int sig)
+static void sig_handler(int sig)
 {
 	switch (sig) {
 	case SIGINT:
@@ -172,10 +177,10 @@ int ping(char *mesh_iface, int argc, char **argv)
 	packet_len = sizeof(struct batadv_icmp_packet);
 
 	memcpy(&icmp_packet_out.dst, dst_mac, ETH_ALEN);
-	icmp_packet_out.header.packet_type = BATADV_ICMP;
-	icmp_packet_out.header.version = BATADV_COMPAT_VERSION;
+	icmp_packet_out.packet_type = BATADV_ICMP;
+	icmp_packet_out.version = BATADV_COMPAT_VERSION;
 	icmp_packet_out.msg_type = BATADV_ECHO_REQUEST;
-	icmp_packet_out.header.ttl = 50;
+	icmp_packet_out.ttl = 50;
 	icmp_packet_out.seqno = 0;
 
 	if (rr) {
@@ -249,8 +254,10 @@ read_packet:
 		case BATADV_ECHO_REPLY:
 			time_delta = end_timer();
 			printf("%zd bytes from %s icmp_seq=%hu ttl=%d time=%.2f ms",
-					read_len, dst_string, ntohs(icmp_packet_in.seqno),
-					icmp_packet_in.header.ttl, time_delta);
+					read_len, dst_string,
+					ntohs(icmp_packet_in.seqno),
+					icmp_packet_in.ttl,
+					time_delta);
 
 			if (read_len == sizeof(struct batadv_icmp_packet_rr)) {
 				if (last_rr_cur == icmp_packet_in.rr_cur
@@ -299,11 +306,12 @@ read_packet:
 			break;
 		case BATADV_PARAMETER_PROBLEM:
 			fprintf(stderr, "Error - the batman adv kernel module version (%d) differs from ours (%d)\n",
-					icmp_packet_in.header.version, BATADV_COMPAT_VERSION);
+				icmp_packet_in.version, BATADV_COMPAT_VERSION);
 			printf("Please make sure to use compatible versions!\n");
 			goto out;
 		default:
-			printf("Unknown message type %d len %zd received\n", icmp_packet_in.msg_type, read_len);
+			printf("Unknown message type %d len %zd received\n",
+			       icmp_packet_in.msg_type, read_len);
 			break;
 		}
 
