@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2013 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2014 B.A.T.M.A.N. contributors:
  *
  * Andreas Langer <an.langer@gmx.de>
  *
@@ -32,12 +32,18 @@
 #include <net/if_arp.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
+#include <netinet/ip6.h>
 #include <netinet/tcp.h>
 #include <netinet/udp.h>
 #include <netinet/ip_icmp.h>
+#include <netinet/icmp6.h>
 #include <netinet/if_ether.h>
+#include <net/ethernet.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <sys/select.h>
+#include <sys/socket.h>
 
-#include "main.h"
 #include "tcpdump.h"
 #include "packet.h"
 #include "bat-hosts.h"
@@ -47,6 +53,8 @@
 #define ETH_P_BATMAN	0x4305
 #endif /* ETH_P_BATMAN */
 
+#define IPV6_MIN_MTU	1280
+
 #define LEN_CHECK(buff_len, check_len, desc) \
 if ((size_t)(buff_len) < (check_len)) { \
 	fprintf(stderr, "Warning - dropping received %s packet as it is smaller than expected (%zu): %zu\n", \
@@ -55,7 +63,7 @@ if ((size_t)(buff_len) < (check_len)) { \
 }
 
 static unsigned short dump_level_all = DUMP_TYPE_BATOGM | DUMP_TYPE_BATICMP | DUMP_TYPE_BATUCAST |
-		DUMP_TYPE_BATBCAST | DUMP_TYPE_BATVIS | DUMP_TYPE_BATFRAG | DUMP_TYPE_BATTT | DUMP_TYPE_NONBAT;
+		DUMP_TYPE_BATBCAST | DUMP_TYPE_BATFRAG | DUMP_TYPE_NONBAT;
 static unsigned short dump_level;
 
 static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed);
@@ -74,9 +82,7 @@ static void tcpdump_usage(void)
 	fprintf(stderr, " \t\t%3d - batman icmp packets\n", DUMP_TYPE_BATICMP);
 	fprintf(stderr, " \t\t%3d - batman unicast packets\n", DUMP_TYPE_BATUCAST);
 	fprintf(stderr, " \t\t%3d - batman broadcast packets\n", DUMP_TYPE_BATBCAST);
-	fprintf(stderr, " \t\t%3d - batman vis packets\n", DUMP_TYPE_BATVIS);
 	fprintf(stderr, " \t\t%3d - batman fragmented packets\n", DUMP_TYPE_BATFRAG);
-	fprintf(stderr, " \t\t%3d - batman tt / roaming packets\n", DUMP_TYPE_BATTT);
 	fprintf(stderr, " \t\t%3d - non batman packets\n", DUMP_TYPE_NONBAT);
 	fprintf(stderr, " \t\t%3d - batman ogm & non batman packets\n", DUMP_TYPE_BATOGM | DUMP_TYPE_NONBAT);
 }
@@ -190,13 +196,181 @@ static void dump_arp(unsigned char *packet_buff, ssize_t buff_len,
 	}
 }
 
-static void dump_ip(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
+static void dump_tcp(const char ip_string[], unsigned char *packet_buff,
+		     ssize_t buff_len, size_t ip6_header_len, char *src_addr,
+		     char *dst_addr)
+{
+	uint16_t tcp_header_len;
+	struct tcphdr *tcphdr;
+
+	LEN_CHECK((size_t)buff_len - ip6_header_len,
+		  sizeof(struct tcphdr), "TCP");
+	tcphdr = (struct tcphdr *)(packet_buff + ip6_header_len);
+	tcp_header_len = tcphdr->doff * 4;
+	printf("%s %s.%i > ", ip_string, src_addr, ntohs(tcphdr->source));
+	printf("%s.%i: TCP, Flags [%c%c%c%c%c%c], length %zu\n",
+		dst_addr, ntohs(tcphdr->dest),
+		(tcphdr->fin ? 'F' : '.'), (tcphdr->syn ? 'S' : '.'),
+		(tcphdr->rst ? 'R' : '.'), (tcphdr->psh ? 'P' : '.'),
+		(tcphdr->ack ? 'A' : '.'), (tcphdr->urg ? 'U' : '.'),
+		(size_t)buff_len - ip6_header_len - tcp_header_len);
+}
+
+static void dump_udp(const char ip_string[], unsigned char *packet_buff,
+		     ssize_t buff_len, size_t ip6_header_len, char *src_addr,
+		     char *dst_addr)
+{
+	struct udphdr *udphdr;
+
+	LEN_CHECK((size_t)buff_len - ip6_header_len, sizeof(struct udphdr),
+		  "UDP");
+	udphdr = (struct udphdr *)(packet_buff + ip6_header_len);
+	printf("%s %s.%i > ", ip_string, src_addr, ntohs(udphdr->source));
+
+	switch (ntohs(udphdr->dest)) {
+	case 67:
+		LEN_CHECK((size_t)buff_len - ip6_header_len -
+			  sizeof(struct udphdr), (size_t) 44, "DHCP");
+		printf("%s.67: BOOTP/DHCP, Request from %s, length %zu\n",
+		       dst_addr,
+		       ether_ntoa_long((struct ether_addr *)(((char *)udphdr) +
+				       sizeof(struct udphdr) + 28)),
+		       (size_t)buff_len - ip6_header_len -
+		       sizeof(struct udphdr));
+		break;
+	case 68:
+		printf("%s.68: BOOTP/DHCP, Reply, length %zu\n", dst_addr,
+		       (size_t)buff_len - ip6_header_len -
+		       sizeof(struct udphdr));
+		break;
+	default:
+		printf("%s.%i: UDP, length %zu\n", dst_addr,
+		       ntohs(udphdr->dest),
+		       (size_t)buff_len - ip6_header_len -
+		       sizeof(struct udphdr));
+		break;
+	}
+}
+
+static void dump_ipv6(unsigned char *packet_buff, ssize_t buff_len,
+		      int time_printed)
+{
+	struct ip6_hdr *iphdr;
+	struct icmp6_hdr *icmphdr;
+
+	char ipv6_addr_src[40], ipv6_addr_dst[40];
+	char ip_saddr[40], ip_daddr[40], nd_nas_target[40];
+	struct nd_neighbor_solicit *nd_neigh_sol;
+	struct nd_neighbor_advert *nd_advert;
+	const char ip_string[] = "IP6";
+
+	iphdr = (struct ip6_hdr *)packet_buff;
+	LEN_CHECK((size_t)buff_len, (size_t)(sizeof(struct ip6_hdr)), "IP6");
+
+	if (!time_printed)
+		print_time();
+
+	inet_ntop(AF_INET6, &(iphdr->ip6_src), ipv6_addr_src, 40);
+	inet_ntop(AF_INET6, &(iphdr->ip6_dst), ipv6_addr_dst, 40);
+
+	switch (iphdr->ip6_nxt) {
+	case IPPROTO_ICMPV6:
+		LEN_CHECK((size_t)buff_len - (size_t)(sizeof(struct ip6_hdr)),
+			  sizeof(struct icmp6_hdr), "ICMPv6");
+		icmphdr = (struct icmp6_hdr *)(packet_buff +
+					       sizeof(struct ip6_hdr));
+
+		printf("IP6 %s > %s ", ipv6_addr_src, ipv6_addr_dst);
+		if (icmphdr->icmp6_type < ICMP6_INFOMSG_MASK &&
+		    (size_t)(buff_len) > IPV6_MIN_MTU) {
+			fprintf(stderr,
+				"Warning - dropping received 'ICMPv6 destination unreached' packet as it is bigger than maximum allowed size (%u): %zu\n",
+				IPV6_MIN_MTU, (size_t)(buff_len));
+			return;
+		}
+
+		printf("ICMP6");
+		switch (icmphdr->icmp6_type) {
+		case ICMP6_DST_UNREACH:
+			switch (icmphdr->icmp6_code) {
+			case ICMP6_DST_UNREACH_NOROUTE:
+				printf(", unreachable route\n");
+				break;
+			case ICMP6_DST_UNREACH_ADMIN:
+				printf(", unreachable prohibited\n");
+				break;
+			case ICMP6_DST_UNREACH_ADDR:
+				printf(", unreachable address\n");
+				break;
+			case ICMP6_DST_UNREACH_BEYONDSCOPE:
+				printf(", beyond scope\n");
+				break;
+			case ICMP6_DST_UNREACH_NOPORT:
+				printf(", unreachable port\n");
+				break;
+			default:
+				printf(", unknown unreach code (%u)\n",
+				       icmphdr->icmp6_code);
+			}
+			break;
+		case ICMP6_ECHO_REQUEST:
+			printf(" echo request, id: %d, seq: %d, length: %hu\n",
+			       icmphdr->icmp6_id, icmphdr->icmp6_seq,
+			       iphdr->ip6_plen);
+			break;
+		case ICMP6_ECHO_REPLY:
+			printf(" echo reply, id: %d, seq: %d, length: %hu\n",
+			       icmphdr->icmp6_id, icmphdr->icmp6_seq,
+			       iphdr->ip6_plen);
+			break;
+		case ICMP6_TIME_EXCEEDED:
+			printf(" time exceeded in-transit, length %zu\n",
+			       (size_t)buff_len - sizeof(struct icmp6_hdr));
+			break;
+		case ND_NEIGHBOR_SOLICIT:
+			nd_neigh_sol = (struct nd_neighbor_solicit *)icmphdr;
+			inet_ntop(AF_INET6, &(nd_neigh_sol->nd_ns_target),
+				  nd_nas_target, 40);
+			printf(" neighbor solicitation, who has %s, length %zd\n",
+			       nd_nas_target, buff_len);
+			break;
+		case ND_NEIGHBOR_ADVERT:
+			nd_advert = (struct nd_neighbor_advert *)icmphdr;
+			inet_ntop(AF_INET6, &(nd_advert->nd_na_target),
+				  nd_nas_target, 40);
+			printf(" neighbor advertisement, tgt is %s, length %zd\n",
+			       nd_nas_target, buff_len);
+			break;
+		default:
+			printf(", destination unreachable, unknown icmp6 type (%u)\n",
+			       icmphdr->icmp6_type);
+			break;
+		}
+		break;
+	case IPPROTO_TCP:
+		inet_ntop(AF_INET6, &(iphdr->ip6_src), ip_saddr, 40);
+		inet_ntop(AF_INET6, &(iphdr->ip6_dst), ip_daddr, 40);
+		dump_tcp(ip_string, packet_buff, buff_len,
+			 sizeof(struct ip6_hdr), ip_saddr, ip_daddr);
+		break;
+	case IPPROTO_UDP:
+		inet_ntop(AF_INET6, &(iphdr->ip6_src), ip_saddr, 40);
+		inet_ntop(AF_INET6, &(iphdr->ip6_dst), ip_daddr, 40);
+		dump_udp(ip_string, packet_buff, buff_len,
+			 sizeof(struct ip6_hdr), ip_saddr, ip_daddr);
+		break;
+	default:
+		printf(" IPv6 unknown protocol: %i\n", iphdr->ip6_nxt);
+	}
+}
+
+static void dump_ip(unsigned char *packet_buff, ssize_t buff_len,
+		    int time_printed)
 {
 	struct iphdr *iphdr, *tmp_iphdr;
-	struct tcphdr *tcphdr;
-	struct udphdr *udphdr, *tmp_udphdr;
+	struct udphdr *tmp_udphdr;
 	struct icmphdr *icmphdr;
-	uint16_t tcp_header_len;
+	const char ip_string[] = "IP";
 
 	iphdr = (struct iphdr *)packet_buff;
 	LEN_CHECK((size_t)buff_len, (size_t)(iphdr->ihl * 4), "IP");
@@ -257,51 +431,16 @@ static void dump_ip(unsigned char *packet_buff, ssize_t buff_len, int time_print
 				(size_t)buff_len - (iphdr->ihl * 4));
 			break;
 		}
-
 		break;
 	case IPPROTO_TCP:
-		tcphdr = (struct tcphdr *)(packet_buff + (iphdr->ihl * 4));
-		tcp_header_len = tcphdr->doff * 4;
-		LEN_CHECK((size_t)buff_len - (iphdr->ihl * 4),
-			  (size_t)tcp_header_len, "TCP");
-
-		printf("IP %s.%i > ", inet_ntoa(*(struct in_addr *)&iphdr->saddr), ntohs(tcphdr->source));
-		printf("%s.%i: TCP, flags [%c%c%c%c%c%c], length %zu\n",
-			inet_ntoa(*(struct in_addr *)&iphdr->daddr), ntohs(tcphdr->dest),
-			(tcphdr->fin ? 'F' : '.'), (tcphdr->syn ? 'S' : '.'),
-			(tcphdr->rst ? 'R' : '.'), (tcphdr->psh ? 'P' : '.'),
-			(tcphdr->ack ? 'A' : '.'), (tcphdr->urg ? 'U' : '.'),
-			(size_t)buff_len - (iphdr->ihl * 4) - tcp_header_len);
+		dump_tcp(ip_string, packet_buff, buff_len, iphdr->ihl * 4,
+			 inet_ntoa(*(struct in_addr *)&iphdr->saddr),
+			 inet_ntoa(*(struct in_addr *)&iphdr->daddr));
 		break;
 	case IPPROTO_UDP:
-		LEN_CHECK((size_t)buff_len - (iphdr->ihl * 4), sizeof(struct udphdr), "UDP");
-
-		udphdr = (struct udphdr *)(packet_buff + (iphdr->ihl * 4));
-		printf("IP %s.%i > ", inet_ntoa(*(struct in_addr *)&iphdr->saddr), ntohs(udphdr->source));
-
-		switch (ntohs(udphdr->dest)) {
-		case 67:
-                        LEN_CHECK((size_t)buff_len - (iphdr->ihl * 4) - sizeof(struct udphdr), (size_t) 44, "DHCP");
-			printf("%s.67: BOOTP/DHCP, Request from %s, length %zu\n",
-				inet_ntoa(*(struct in_addr *)&iphdr->daddr),
-				ether_ntoa_long((struct ether_addr *)(((char *)udphdr) + sizeof(struct udphdr) + 28)),
-				(size_t)buff_len - (iphdr->ihl * 4) - sizeof(struct udphdr));
-			break;
-		case 68:
-			printf("%s.68: BOOTP/DHCP, Reply, length %zu\n",
-				inet_ntoa(*(struct in_addr *)&iphdr->daddr),
-				(size_t)buff_len - (iphdr->ihl * 4) - sizeof(struct udphdr));
-			break;
-		default:
-			printf("%s.%i: UDP, length %zu\n",
-				inet_ntoa(*(struct in_addr *)&iphdr->daddr), ntohs(udphdr->dest),
-				(size_t)buff_len - (iphdr->ihl * 4) - sizeof(struct udphdr));
-			break;
-		}
-
-		break;
-	case IPPROTO_IPV6:
-		printf("IP6: not implemented yet\n");
+		dump_udp(ip_string, packet_buff, buff_len, iphdr->ihl * 4,
+			 inet_ntoa(*(struct in_addr *)&iphdr->saddr),
+			 inet_ntoa(*(struct in_addr *)&iphdr->daddr));
 		break;
 	default:
 		printf("IP unknown protocol: %i\n", iphdr->protocol);
@@ -328,70 +467,6 @@ static void dump_vlan(unsigned char *packet_buff, ssize_t buff_len, int read_opt
 	parse_eth_hdr(packet_buff + 4, buff_len - 4, read_opt, time_printed);
 }
 
-static void dump_batman_tt(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
-{
-	struct batadv_tt_query_packet *tt_query_packet;
-	char *tt_desc, *tt_data, tt_type;
-
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_tt_query_packet), "BAT TT");
-
-	tt_query_packet = (struct batadv_tt_query_packet *)(packet_buff + sizeof(struct ether_header));
-
-	if (!time_printed)
-		print_time();
-
-	switch (tt_query_packet->flags & BATADV_TT_QUERY_TYPE_MASK) {
-	case BATADV_TT_REQUEST:
-		tt_desc = "request";
-		tt_data = "crc";
-		tt_type = 'Q';
-		break;
-	case BATADV_TT_RESPONSE:
-		tt_desc = "response";
-		tt_data = "entries";
-		tt_type = 'P';
-		break;
-	default:
-		tt_desc = "unknown";
-		tt_data = "unknown";
-		tt_type = '?';
-		break;
-	}
-
-	printf("BAT %s > ",
-	       get_name_by_macaddr((struct ether_addr *)tt_query_packet->src, read_opt));
-
-	printf("%s: TT %s, ttvn %d, %s %u, ttl %2d, v %d, flags [%c%c], length %zu\n",
-	       get_name_by_macaddr((struct ether_addr *)tt_query_packet->dst, read_opt),
-	       tt_desc, tt_query_packet->ttvn, tt_data, ntohs(tt_query_packet->tt_data),
-	       tt_query_packet->header.ttl, tt_query_packet->header.version,
-	       tt_type, (tt_query_packet->flags & BATADV_TT_FULL_TABLE ? 'F' : '.'),
-	       (size_t)buff_len - sizeof(struct ether_header));
-}
-
-static void dump_batman_roam(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
-{
-	struct batadv_roam_adv_packet *roam_adv_packet;
-
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_roam_adv_packet), "BAT ROAM");
-
-	roam_adv_packet = (struct batadv_roam_adv_packet *)(packet_buff + sizeof(struct ether_header));
-
-	if (!time_printed)
-		print_time();
-
-	printf("BAT %s > ",
-	       get_name_by_macaddr((struct ether_addr *)roam_adv_packet->src, read_opt));
-
-	printf("%s: ROAM, ",
-	       get_name_by_macaddr((struct ether_addr *)roam_adv_packet->dst, read_opt));
-
-	printf("client %s, ttl %2d, v %d, length %zu\n",
-	       get_name_by_macaddr((struct ether_addr *)roam_adv_packet->client, read_opt),
-	       roam_adv_packet->header.ttl, roam_adv_packet->header.version,
-	       (size_t)buff_len - sizeof(struct ether_header));
-}
-
 static void dump_batman_iv_ogm(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct ether_header *ether_header;
@@ -408,15 +483,13 @@ static void dump_batman_iv_ogm(unsigned char *packet_buff, ssize_t buff_len, int
 	printf("BAT %s: ",
 	       get_name_by_macaddr((struct ether_addr *)batman_ogm_packet->orig, read_opt));
 
-	printf("OGM IV via neigh %s, seq %u, tq %3d, ttvn %d, ttcrc %hu, ttl %2d, v %d, flags [%c%c%c%c%c], length %zu\n",
+	printf("OGM IV via neigh %s, seq %u, tq %3d, ttl %2d, v %d, flags [%c%c%c], length %zu\n",
 	       get_name_by_macaddr((struct ether_addr *)ether_header->ether_shost, read_opt),
-	       ntohl(batman_ogm_packet->seqno), batman_ogm_packet->tq, batman_ogm_packet->ttvn,
-	       ntohs(batman_ogm_packet->tt_crc), batman_ogm_packet->header.ttl, batman_ogm_packet->header.version,
+	       ntohl(batman_ogm_packet->seqno), batman_ogm_packet->tq,
+	       batman_ogm_packet->ttl, batman_ogm_packet->version,
 	       (batman_ogm_packet->flags & BATADV_NOT_BEST_NEXT_HOP ? 'N' : '.'),
 	       (batman_ogm_packet->flags & BATADV_DIRECTLINK ? 'D' : '.'),
-	       (batman_ogm_packet->flags & BATADV_VIS_SERVER ? 'V' : '.'),
 	       (batman_ogm_packet->flags & BATADV_PRIMARIES_FIRST_HOP ? 'F' : '.'),
-	       (batman_ogm_packet->gw_flags ? 'G' : '.'),
 	       (size_t)buff_len - sizeof(struct ether_header));
 }
 
@@ -432,7 +505,8 @@ static void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int r
 	if (!time_printed)
 		print_time();
 
-	printf("BAT %s > ", get_name_by_macaddr((struct ether_addr *)icmp_packet->orig, read_opt));
+	printf("BAT %s > ",
+	       get_name_by_macaddr((struct ether_addr *)icmp_packet->orig, read_opt));
 
 	name = get_name_by_macaddr((struct ether_addr *)icmp_packet->dst, read_opt);
 
@@ -440,24 +514,25 @@ static void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int r
 	case BATADV_ECHO_REPLY:
 		printf("%s: ICMP echo reply, id %hhu, seq %hu, ttl %2d, v %d, length %zu\n",
 			name, icmp_packet->uid, ntohs(icmp_packet->seqno),
-			icmp_packet->header.ttl, icmp_packet->header.version,
+			icmp_packet->ttl, icmp_packet->version,
 			(size_t)buff_len - sizeof(struct ether_header));
 		break;
 	case BATADV_ECHO_REQUEST:
 		printf("%s: ICMP echo request, id %hhu, seq %hu, ttl %2d, v %d, length %zu\n",
 			name, icmp_packet->uid, ntohs(icmp_packet->seqno),
-			icmp_packet->header.ttl, icmp_packet->header.version,
+			icmp_packet->ttl, icmp_packet->version,
 			(size_t)buff_len - sizeof(struct ether_header));
 		break;
 	case BATADV_TTL_EXCEEDED:
 		printf("%s: ICMP time exceeded in-transit, id %hhu, seq %hu, ttl %2d, v %d, length %zu\n",
 			name, icmp_packet->uid, ntohs(icmp_packet->seqno),
-			icmp_packet->header.ttl, icmp_packet->header.version,
+			icmp_packet->ttl, icmp_packet->version,
 			(size_t)buff_len - sizeof(struct ether_header));
 		break;
 	default:
 		printf("%s: ICMP type %hhu, length %zu\n",
-			name, icmp_packet->msg_type, (size_t)buff_len - sizeof(struct ether_header));
+			name, icmp_packet->msg_type,
+			(size_t)buff_len - sizeof(struct ether_header));
 		break;
 	}
 }
@@ -482,7 +557,7 @@ static void dump_batman_ucast(unsigned char *packet_buff, ssize_t buff_len, int
 
 	printf("%s: UCAST, ttvn %d, ttl %hhu, ",
 	       get_name_by_macaddr((struct ether_addr *)unicast_packet->dest, read_opt),
-	       unicast_packet->ttvn, unicast_packet->header.ttl);
+	       unicast_packet->ttvn, unicast_packet->ttl);
 
 	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct batadv_unicast_packet),
 		      buff_len - ETH_HLEN - sizeof(struct batadv_unicast_packet),
@@ -516,35 +591,6 @@ static void dump_batman_bcast(unsigned char *packet_buff, ssize_t buff_len, int
 		      read_opt, time_printed);
 }
 
-static void dump_batman_frag(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
-{
-	struct batadv_unicast_frag_packet *unicast_frag_packet;
-
-	LEN_CHECK((size_t)buff_len - ETH_HLEN, sizeof(struct batadv_unicast_frag_packet), "BAT FRAG");
-	LEN_CHECK((size_t)buff_len - ETH_HLEN - sizeof(struct batadv_unicast_frag_packet), (size_t)ETH_HLEN, "BAT FRAG (unpacked)");
-
-	unicast_frag_packet = (struct batadv_unicast_frag_packet *)(packet_buff + ETH_HLEN);
-
-	if (!time_printed)
-		time_printed = print_time();
-
-	printf("BAT %s > ",
-	       get_name_by_macaddr((struct ether_addr *)unicast_frag_packet->orig, read_opt));
-
-	printf("%s: FRAG, seq %hu, ttvn %d, ttl %hhu, flags [%c%c], ",
-	       get_name_by_macaddr((struct ether_addr *)unicast_frag_packet->dest, read_opt),
-	       ntohs(unicast_frag_packet->seqno), unicast_frag_packet->ttvn, unicast_frag_packet->header.ttl,
-	       (unicast_frag_packet->flags & BATADV_UNI_FRAG_HEAD ? 'H' : '.'),
-	       (unicast_frag_packet->flags & BATADV_UNI_FRAG_LARGETAIL ? 'L' : '.'));
-
-	if (unicast_frag_packet->flags & BATADV_UNI_FRAG_HEAD)
-		parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct batadv_unicast_frag_packet),
-			      buff_len - ETH_HLEN - sizeof(struct batadv_unicast_frag_packet),
-			      read_opt, time_printed);
-	else
-		printf("length %zu\n", (size_t)buff_len - ETH_HLEN - sizeof(struct batadv_unicast_frag_packet));
-}
-
 static void dump_batman_4addr(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct ether_header *ether_header;
@@ -566,7 +612,7 @@ static void dump_batman_4addr(unsigned char *packet_buff, ssize_t buff_len, int
 	printf("%s: 4ADDR, subtybe %hhu, ttvn %d, ttl %hhu, ",
 	       get_name_by_macaddr((struct ether_addr *)unicast_4addr_packet->u.dest, read_opt),
 	       unicast_4addr_packet->subtype, unicast_4addr_packet->u.ttvn,
-	       unicast_4addr_packet->u.header.ttl);
+	       unicast_4addr_packet->u.ttl);
 
 	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct batadv_unicast_4addr_packet),
 		      buff_len - ETH_HLEN - sizeof(struct batadv_unicast_4addr_packet),
@@ -590,6 +636,11 @@ static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read
 		if ((dump_level & DUMP_TYPE_NONBAT) || (time_printed))
 			dump_ip(packet_buff + ETH_HLEN, buff_len - ETH_HLEN, time_printed);
 		break;
+	case ETH_P_IPV6:
+		if ((dump_level & DUMP_TYPE_NONBAT) || (time_printed))
+			dump_ipv6(packet_buff + ETH_HLEN, buff_len - ETH_HLEN,
+				  time_printed);
+		break;
 	case ETH_P_8021Q:
 		if ((dump_level & DUMP_TYPE_NONBAT) || (time_printed))
 			dump_vlan(packet_buff, buff_len, read_opt, time_printed);
@@ -598,10 +649,10 @@ static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read
 		batman_ogm_packet = (struct batadv_ogm_packet *)(packet_buff + ETH_HLEN);
 
 		if ((read_opt & COMPAT_FILTER) &&
-		    (batman_ogm_packet->header.version != BATADV_COMPAT_VERSION))
+		    (batman_ogm_packet->version != BATADV_COMPAT_VERSION))
 			return;
 
-		switch (batman_ogm_packet->header.packet_type) {
+		switch (batman_ogm_packet->packet_type) {
 		case BATADV_IV_OGM:
 			if (dump_level & DUMP_TYPE_BATOGM)
 				dump_batman_iv_ogm(packet_buff, buff_len, read_opt, time_printed);
@@ -618,28 +669,12 @@ static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read
 			if (dump_level & DUMP_TYPE_BATBCAST)
 				dump_batman_bcast(packet_buff, buff_len, read_opt, time_printed);
 			break;
-		case BATADV_VIS:
-			if (dump_level & DUMP_TYPE_BATVIS)
-				fprintf(stderr, "Warning - batman vis packet received: function not implemented yet\n");
-			break;
-		case BATADV_UNICAST_FRAG:
-			if (dump_level & DUMP_TYPE_BATFRAG)
-				dump_batman_frag(packet_buff, buff_len, read_opt, time_printed);
-			break;
-		case BATADV_TT_QUERY:
-			if (dump_level & DUMP_TYPE_BATTT)
-				dump_batman_tt(packet_buff, buff_len, read_opt, time_printed);
-			break;
-		case BATADV_ROAM_ADV:
-			if (dump_level & DUMP_TYPE_BATTT)
-				dump_batman_roam(packet_buff, buff_len, read_opt, time_printed);
-			break;
 		case BATADV_UNICAST_4ADDR:
 			if (dump_level & DUMP_TYPE_BATUCAST)
 				dump_batman_4addr(packet_buff, buff_len, read_opt, time_printed);
 			break;
 		default:
-			fprintf(stderr, "Warning - packet contains unknown batman packet type: 0x%02x\n", batman_ogm_packet->header.packet_type);
+			fprintf(stderr, "Warning - packet contains unknown batman packet type: 0x%02x\n", batman_ogm_packet->packet_type);
 			break;
 		}
 
